from . import validators as v
from . import KwargsMixin
from .excs import *
import datetime


class _base_ft(KwargsMixin):
	pass

class string_ft(_base_ft):
	maxlen = None
	minlen = None
	required = False

	def get_pre_validation_data(self, n, data, default=None):
		a = data.get(n, None)
		if len(a) == 0 and default != None:
			a = default
		return a

	def get_post_validation_data(self, n, data, default=None):
		return self.get_pre_validation_data(n, data, default=default)

	def validators(self):
		vs = []
		if self.required:
			vs.append( v.required_v(required=self.required) )
		if self.maxlen != None:
			vs.append( v.maxlen_v(maxlen=self.maxlen) )
		if self.minlen != None:
			vs.append( v.minlen_v(minlen=self.minlen) )
		return vs


class int_ft(_base_ft):
	required = False

	def get_pre_validation_data(self, n, data, default=None):
		value = data.get(n, None)
		if len(str(value)) == 0:
			value == default if default != None else None
		return value

	def get_post_validation_data(self, n, data, default=None):
		value = self.get_pre_validation_data(n, data, default=default)
		if value in ["", None]:
			return None
		return int(str(value))

	def validators(self):
		vs = [ v.int_v(), ]
		if self.required:
			vs.append(v.required_v(required=self.required))
		return vs


class bool_ft(_base_ft):
	question = ''

	def get_pre_validation_data(self, n, data, default=None):
		value = data.get(n, None)
		if value in ["", None] and default != None:
			value = default

		# make sure value is either "on" or "off"
		if value == None:
			value = 'off'

		return value

	def get_post_validation_data(self, n, data, default=None):
		return self.get_pre_validation_data(n, data, default)

	def validators(self):
		vs = [ v.bool_value_v(), ]
		return vs


class choice_ft(_base_ft):
	choices = ()
	required = False

	def get_pre_validation_data(self, n, data, default=None):
		value = data.get(n, None)
		if value == None and default != None:
			value = default
		return value

	def get_post_validation_data(self, n, data, default=None):
		return self.get_pre_validation_data(n, data, default)

	def validators(self):
		vs = [ v.choice_choices_v(choices=self.choices), ]
		if self.required:
			vs.append( v.choice_required_v(required=self.required) )
		return vs

class multi_choice_ft(_base_ft):
	choices = ()
	with_parentheses = False # jQuery and some libraries adds "[]" to sent names

	def get_pre_validation_data(self, n, data, default=None):
		value = []
		if self.with_parentheses:
			value = data.getlist('' + n + '[]', [])
		else:
			value = data.getlist(n, [])

		if value == [] and default != None:
			value = default

		return value

	def get_post_validation_data(self, n, data, default=None):
		return self.get_pre_validation_data(n, data, default=default)

	def validators(self):
		vs = [ v.multichoice_choices_v(choices=self.choices), ]
		return vs


class date_ft(_base_ft):
	required = False
	input_formats = [
		'%Y-%m-%d',			# '2006-10-25'
		'%m/%d/%Y',			# '10/25/2006'
		'%m/%d/%y'			# '10/25/06'
	]

	def get_pre_validation_data(self, n, data, default=None):
		value = data.get(n, None)
		if value in ["", None] and default != None:
			value = default

		return value

	def get_post_validation_data(self, n, data, default=None):
		value = self.get_pre_validation_data(n, data, default)

		# attimes the value would be to a "datetime.date" instance if set from the default
		if isinstance(value, datetime.date):
			return value

		if value in ["", None]:
			return None

		for a in self.input_formats:
			try:
				return datetime.datetime.strptime(value, a).date()
			except ValueError:
				continue

	def validators(self):
		vs = [ v.date_format_v(input_formats=self.input_formats), ]
		if self.required:
			vs.append(v.date_required_v(required=self.required))
		return vs


class time_ft(_base_ft):
	required = False
	input_formats = [
		'%H:%M:%S',     # '14:30:59'
		'%H:%M',        # '14:30'
	]

	def get_pre_validation_data(self, n, data, default=None):
		value = data.get(n, None)
		if value in ["", None] and default != None:
			value = default
		return value

	def get_post_validation_data(self, n, data, default=None):
		value = self.get_pre_validation_data(n, data, default=default)

		# attimes the value would be to a "datetime.time" instance if set from the default
		if isinstance(value, datetime.time):
			return value

		if value in ["", None]:
			return None

		for a in self.input_formats:
			try:
				return datetime.datetime.strptime(value, a).time()
			except ValueError:
				continue

	def validators(self):
		vs = [ v.time_format_v(input_formats=self.input_formats), ]
		if self.required:
			vs.append(v.time_required_v(required=self.required))
		return vs


class datetime_ft(_base_ft):
	required = False
	input_formats = [
		'%Y-%m-%d %H:%M:%S',     # '2006-10-25 14:30:59'
		'%Y-%m-%d %H:%M',        # '2006-10-25 14:30'
		'%Y-%m-%d',              # '2006-10-25'
		'%m/%d/%Y %H:%M:%S',     # '10/25/2006 14:30:59'
		'%m/%d/%Y %H:%M',        # '10/25/2006 14:30'
		'%m/%d/%Y',              # '10/25/2006'
		'%m/%d/%y %H:%M:%S',     # '10/25/06 14:30:59'
		'%m/%d/%y %H:%M',        # '10/25/06 14:30'
		'%m/%d/%y'               # '10/25/06'
	]

	def get_pre_validation_data(self, n, data, default=None):
		value = data.get(n, None)
		if value in ["", None] and default != None:
			value = default
		return value

	def get_post_validation_data(self, n, data, default=None):
		value = self.get_pre_validation_data(n, data, default=default)

		# attimes the value would be to a "datetime.datetime" instance if set from the default
		if isinstance(value, datetime.datetime):
			return value

		if value in ["", None]:
			return None

		for a in self.input_formats:
			try:
				return datetime.datetime.strptime(value, a)
			except ValueError:
				continue

	def validators(self):
		vs = [ v.datetime_format_v(input_formats=self.input_formats), ]
		if self.required:
			vs.append(v.datetime_required_v(required=self.required))
		return vs


