from . import KwargsMixin
import datetime


class _base_v(KwargsMixin):
	pass

class maxlen_v(_base_v):
	errcode = 10
	msg = u"field length exceeds the maximum length ({maxlen})"

	def validate(self, value):
		if len(value) > self.maxlen:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(maxlen=self.maxlen, errcode=self.errcode, value=value)

class minlen_v(_base_v):
	errcode = 11
	msg = u"field length is less than the minimum length ({minlen})"

	def validate(self, value):
		if len(value) < self.minlen:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(minlen=self.minlen, errcode=self.errcode, value=value)


class maxval_v(_base_v):
	errcode = 12
	msg = u"field value is greater than maximum value ({maxval})"

	def validate(self, value):
		if int(str(value)) > self.maxval:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(maxval=self.maxval, errcode=self.errcode, value=value)

class minval_v(_base_v):
	errcode = 13
	msg = u"field value is less than the minimum value ({minval})"

	def validate(self, value):
		if int(str(value)) < self.minval:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(minval=self.minval, errcode=self.errcode, value=value)


class int_v(_base_v):
	errcode = 14
	msg = u"input value not convertible to int"

	def validate(self, value):

		if value in ["", None]:
			return

		try:
			value = int(str(value))
		except (ValueError, TypeError):
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode, value=value)


class required_v(_base_v):
	errcode = 15
	msg = u"this field is required"

	def validate(self, value):
		if len(str(value)) == 0 or value == None:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode, value=value)


class bool_value_v(_base_v):
	errcode = 16
	msg = u"this field was given an unacceptable input"

	def validate(self, value):
		if value not in [ 'on', 'off',]:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class choice_choices_v(_base_v):
	errcode = 17
	msg = u"this field was given an unacceptable input"

	def validate(self, value):
		a = False
		for b, c in self.choices:
			if value == b:
				a = True
				break

		if value in ['', None]:   # if the value is not selected
			return

		if not a:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class multichoice_choices_v(_base_v):
	errcode = 18
	msg = u"this field was given unacceptable input(s)"

	def validate(self, value):
		a = False
		for b in value:
			e = [ c for c, d in self.choices ]
			if b not in e:
				return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class choice_required_v(_base_v):
	errcode = 19
	msg = u"This field is required"

	def validate(self, value):
		if value == None:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class date_required_v(_base_v):
	errcode = 20
	msg = u"This field is required"

	def validate(self, value):
		if value == "":
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class time_required_v(_base_v):
	errcode = 21
	msg = u"This field is required"

	def validate(self, value):
		if value == "":
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class datetime_required_v(_base_v):
	errcode = 22
	msg = u"This field is required"

	def validate(self, value):
		if value == "":
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class date_format_v(_base_v):
	errcode = 23
	msg = u"this field was given unacceptable input(s)"

	def validate(self, value):
		if isinstance(value, datetime.date):
			return

		if value in ["", None]:
			return

		c = False
		for a in self.input_formats:
			try:
				b = datetime.datetime.strptime(value, a)
				c = True
			except ValueError:
				continue

			if c == True:
				return

		return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class time_format_v(_base_v):
	errcode = 24
	msg = u"this field was given an unacceptable input"

	def validate(self, value):
		if isinstance(value, datetime.time):
			return

		if value in ["", None]:
			return

		c = False
		for a in self.input_formats:
			try:
				b = datetime.datetime.strptime(value, a).time()
				c = True
			except ValueError:
				continue

			if c == True:
				return

		return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class datetime_format_v(_base_v):
	errcode = 25
	msg = u"this field was given an unacceptable input"

	def validate(self, value):
		if isinstance(value, datetime.datetime):
			return

		if value in ["", None]:
			return

		c = False
		for a in self.input_formats:
			try:
				b = datetime.datetime.strptime(value, a)
				c = True
			except ValueError:
				continue

			if c == True:
				return

		return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)



